@@ -67,8 +67,7 @@ sub parse_xml ($list, $map) {
     my $manpage = sub ($t, $elt) {
         my $man = $elt->first_child('refentrytitle')->text;
         my $nb = $elt->first_child('manvolnum')->text;
-        my $url = "https://manpages.debian.org/cgi-bin/man.cgi?query=$man&sektion=$nb&manpath=Debian+unstable+sid";
-        $elt->set_text( qq!L<$man($nb)|"$url">!);
+        $elt->set_text( qq!L<$man($nb)>!);
     };
 
     my $condition_variable = sub  ($t, $elt) {
@@ -79,25 +78,40 @@ sub parse_xml ($list, $map) {
         foreach my $var_elt (@var_list) {
             my $var_name = $var_elt->text;
             my ($var_doc_elt) = $listitem->get_xpath(qq!./para/varname[string()="$var_name"]!);
-            #say "condition_variable $var_name found at ",$var_doc_elt->path;
-            my ($name, $extra_info) = split '=', $var_name, 2;
-            say $file->basename(".xml").": class $config_class element $name, trashed $extra_info" if $extra_info;
+            # say "condition_variable $var_name found at ",$var_doc_elt->path;
+            my ($name, $extra_info) = $var_name =~ /C<([\w-]+)=([^>]*)>/ ;
+            die "Error: cannot extract parameter name from '$var_name'" unless defined $name;
             my $desc = join ("\n\n", $pre_doc, $var_doc_elt->parent->text, $post_doc);
             push $data{element}->@*, [$config_class => $name => $desc => $extra_info];
         }
     };
 
     my $variable = sub  ($t, $elt) {
-        return $condition_variable->($t, $elt) if $elt->first_child_text('term') =~ /^Condition/;
+        return $condition_variable->($t, $elt) if $elt->first_child_text('term') =~ /^C<Condition/;
         my $desc = $elt->first_child('listitem')->text;
-        $desc =~ s/(\w+)=/C<$1>/g;
+
+        # detect deprecated param and what replaces them
+        my @supersedes ;
+        if ($desc =~ /settings? (?:are|is) deprecated. Use ([\w=\s,]+)./) {
+            my $capture = $1;
+            @supersedes = $capture =~ /(\w+)=/g;
+        }
+
+        # apply substitution only on variable names (FooBar=). The regexp must test
+        # for capital letter and C<..> to avoid breaking URL parameters
+        $desc =~ s/C<([A-Z]\w+)=>/C<$1>/g;
+
+        # detect verbatim parts setup with programlisting tag
+        $desc =~ s/^\+-\+/    /gm;
 
         foreach my $term_elt ($elt->children('term')) {
             my $varname = $term_elt->first_child('varname')->text;
-            my ($name, $extra_info) = split '=', $varname, 2;
-            say $file->basename(".xml").": class $config_class element $name, trashed $extra_info" if $extra_info;
+            my ($name, $extra_info) = $varname =~ /C<([\w-]+)=([^>]*)>/ ;
 
-            push $data{element}->@*, [$config_class => $name => $desc => $extra_info];
+            die "Error: cannot extract parameter name from '$varname'" unless defined $name;
+
+            # we hope that deprecated items are listed in the same order with the new items
+            push $data{element}->@*, [$config_class => $name => $desc => $extra_info => shift @supersedes ];
         }
     };
 
@@ -112,12 +126,25 @@ sub parse_xml ($list, $map) {
             $set_config_class->($1) ;
         }
     };
+    my $turn_to_pod_c = sub { my $t = $_->text(); $_->set_text("C<$t>");};
     my $twig = XML::Twig->new (
         twig_handlers => {
             'refsect1/title' => $parse_sub_title,
             'refsect1[string(title)=~ /Description/]/para' => $desc,
             'citerefentry' => $manpage,
-            'literal' => sub { my $t = $_->text(); $_->set_text("C<$t>");},
+            'literal' => $turn_to_pod_c,
+            'option' => $turn_to_pod_c,
+            'constant' => $turn_to_pod_c,
+            # this also remove the indentation of programlisting
+            # element,
+            'para' => sub { $_->subs_text(qr/\n\s+/,"\n"); 1;},
+            # hack: use my own tag which is removed later to create
+            # code block. Can't directly indent text as the content of
+            # para element is also indented (and cleanup above)
+            'programlisting' => sub {my $t = $_->text(); $t =~ s/\n\s*/\n+-+/g; $_->set_text("\n\n+-+$t\n\n");},
+            # varname handling is done before the variable handling
+            # below
+            'varname' => $turn_to_pod_c,
             'refsect1[string(title)=~ /Options/]/variablelist/varlistentry' => $variable,
         }
     );
@@ -132,7 +159,9 @@ sub parse_xml ($list, $map) {
 }
 
 
-sub setup_element ($meta_root, $config_class, $element, $desc, $extra_info) {
+sub setup_element ($meta_root, $config_class, $element, $desc, $extra_info, $supersedes) {
+
+    my @log;
 
     my $obj = $meta_root->grab(
         step => "class:$config_class element:$element",
@@ -147,55 +176,93 @@ sub setup_element ($meta_root, $config_class, $element, $desc, $extra_info) {
         = $desc =~ /Takes a boolean argument or/ ? 'enum'
         : $desc =~ /Takes an? (boolean|integer)/ ? $1
         : $desc =~ /Takes time \(in seconds\)/   ? 'integer'
+        : $desc =~ /allowed range/i              ? 'integer'
         : $desc =~ /Takes one of/                ? 'enum'
         : $extra_info =~ /\w\|\w/                ? 'enum'
         :                                          'uniline';
 
-    my ($min, $max) = ($desc =~ /Takes an integer between ([-\d]+) (?:\([\w\s]+\))? and ([-\d+])/) ;
+    if ($extra_info and $value_type ne 'enum') {
+        push @log, "did not use extra info: $extra_info" unless
+            scalar grep {$extra_info eq $_} qw/weight range/;
+    }
+
+    my ($min, $max);
+    if ($desc =~ /Takes an integer between ([-\d]+) (?:\([\w\s]+\))? and ([-\d]+)/) {
+        ($min, $max) = ($1, $2);
+        push @log, "integer between $min and $max";
+    }
+    if ($desc =~ /allowed range is ([-\d]+) to ([-\d]+)/) {
+        ($min, $max) = ($1, $2);
+        push @log, "integer range is $min to $max";
+    }
 
     my @load ;
+    my @load_extra;
+
+    if ($value_type eq 'integer' and $desc =~ /usual suffixes K/) {
+        $value_type = 'uniline';
+        push @load_extra , q!match="^\d+(?i)[KMG]$"!;
+    }
 
     push @load, qw/type=list cargo/ if $element =~ /^Exec/ or $desc =~ /may be specified more than once/;
 
     push @load, 'type=leaf', "value_type=$value_type";
 
-    push @load, 'write_as=no,yes' if $value_type eq 'boolean';
+    push @load_extra, 'write_as=no,yes' if $value_type eq 'boolean';
+
 
     if ($value_type eq 'enum') {
         my @choices;
         if ($extra_info =~ /\w\|\w/) {
             @choices = split /\|/, $extra_info ;
         }
-        elsif ($desc =~ /Takes a boolean argument or (?:C<)?([\w-]+)/) {
-            @choices = ('no','yes',$1);
+        elsif ($desc =~ /Takes a boolean argument or /) {
+            my ($choices) = ($desc =~ /Takes a boolean argument or (?:the )?(?:special values|architecture identifiers\s*)?([^.]+?)\./);
+            @choices = ('no','yes');
+            push @choices, extract_choices($choices);
             push @load, qw/replace:false=no replace:true=yes replace:0=no replace:1=yes/;
         }
 
         if ($desc =~ /Takes one of/) {
             my ($choices) = ($desc =~ /Takes one of ([^.]+?)(?:\.|to test)/);
-            $choices =~ s/\(the default\)//g;
-            $choices =~ s/\b(or|and)\b/,/g;
-            $choices =~ s/\s//g;
-            $choices =~ s/C<(\w+)>/$1/g;
-            $choices =~ s/,+/,/g;
-            say "set choice $choices";
-            @choices = split /,/, $choices;
+            @choices = extract_choices($choices);
         }
 
         die "Error in $config_class: cannot find the values of $element enum type\n"
             unless @choices;
-        push @load, 'choice='.join(',',@choices);
+        push @log, "enum choices are '".join("', '", @choices)."'";
+        push @load_extra, 'choice='.join(',',@choices);
     }
 
 
-    push @load, "min=$min" if defined $min;
-    push @load, "max=$max" if defined $max;
+    push @load_extra, "min=$min" if defined $min;
+    push @load_extra, "max=$max" if defined $max;
 
-    $obj->load(step => \@load);
+    if ($value_type eq 'integer' and $desc =~ /defaults? (?:to|is) (\d+)/i) {
+        push @load_extra, "default=$1" ;
+    }
 
+    if ($supersedes) {
+        push @load_extra, "status=deprecated";
+
+        push @log, "deprecated in favor of $supersedes";
+        # put migration in place for the other element
+        my $new = $meta_root->grab(
+            step => "class:$config_class element:$supersedes",
+            autoadd => 1
+        );
+        $new->load(steps => qq!migrate_from variables:old="- $element" formula="\$old"!);
+    }
+    $obj->load(step => [@load, @load_extra]);
+
+    say "class $config_class element $element:\n\t".join("\n\t", @log) if @log;
     return $obj;
 }
 
+sub extract_choices($choices) {
+    return $choices =~ /C<([\w\-+]+)>/g;
+}
+
 my $data = parse_xml([@list, @service_list], \%map) ;
 
 # Itself constructor returns an object to read or write the data
@@ -223,6 +290,7 @@ foreach my $config_class (keys $data->{class}->%*) {
     my $desc_ref = $data->{class}{$config_class};
     my $desc_text = join("\n\n",$desc_ref->@*);
     $desc_text =~ s/^\s+//gm;
+    $desc_text =~ s/C<([A-Z]\w+)=>/C<$1>/g;
 
     $desc_text.="\nThis configuration class was generated from systemd documentation.\n"
         ."by L<parse-man.pl|https://github.com/dod38fr/config-model-systemd/contrib/parse-man.pl>\n";
@@ -241,17 +309,9 @@ foreach my $config_class (keys $data->{class}->%*) {
 }
 
 foreach my $cdata ($data->{element}->@*) {
-    my ($config_class, $element, $desc, $extra_info) = $cdata->@*;
+    my ($config_class, $element, $desc, $extra_info, $supersedes) = $cdata->@*;
 
-    my $obj = setup_element ($meta_root, $config_class, $element, $desc, $extra_info);
-
-    # cleanup one utf8 characters (aka \x{2014}). This can be removed once Config::Model 2.084
-    # is released: generated pod will declare utf8 encoding
-    $desc =~ s/—/--/g;
-    $desc =~ s/ / /g; # not exactly an underscore
-
-    # cleanup empty lines
-    $desc =~ s/^\s+//gm;
+    my $obj = setup_element ($meta_root, $config_class, $element, $desc, $extra_info, $supersedes);
 
     $obj->fetch_element("description")->store($desc);
 }
